// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2020 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package models

import (
	"code.vikunja.io/api/pkg/db"
	"code.vikunja.io/api/pkg/user"
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestSavedFilter_getListIDFromFilter(t *testing.T) {
	t.Run("normal", func(t *testing.T) {
		assert.Equal(t, int64(-2), getListIDFromSavedFilterID(1))
	})
	t.Run("invalid", func(t *testing.T) {
		assert.Equal(t, int64(0), getListIDFromSavedFilterID(-1))
	})
}

func TestSavedFilter_getFilterIDFromListID(t *testing.T) {
	t.Run("normal", func(t *testing.T) {
		assert.Equal(t, int64(1), getSavedFilterIDFromListID(-2))
	})
	t.Run("invalid", func(t *testing.T) {
		assert.Equal(t, int64(0), getSavedFilterIDFromListID(2))
	})
}

func TestSavedFilter_Create(t *testing.T) {
	db.LoadAndAssertFixtures(t)
	sf := &SavedFilter{
		Title:       "test",
		Description: "Lorem Ipsum dolor sit amet",
		Filters:     &TaskCollection{}, // Empty filter
	}

	u := &user.User{ID: 1}
	err := sf.Create(u)
	assert.NoError(t, err)
	assert.Equal(t, u.ID, sf.OwnerID)
	db.AssertDBExists(t, "saved_filters", map[string]interface{}{
		"title":       "test",
		"description": "Lorem Ipsum dolor sit amet",
		"filters":     "{\"sort_by\":null,\"order_by\":null,\"filter_by\":null,\"filter_value\":null,\"filter_comparator\":null,\"filter_concat\":\"\",\"filter_include_nulls\":false}",
		"owner_id":    1,
	})
}

func TestSavedFilter_ReadOne(t *testing.T) {
	// Also test when getting a list which is actually a saved filter it returns the saved filter
	// Also test getting tasks from a saved filter through the task collection as part of a normal task
	// Also test rights (maybe not here)
}

func TestSavedFilter_Update(t *testing.T) {
	db.LoadAndAssertFixtures(t)
	sf := &SavedFilter{
		ID:          1,
		Title:       "NewTitle",
		Description: "", // Explicitly reset the description
		Filters:     &TaskCollection{},
	}
	err := sf.Update()
	assert.NoError(t, err)
	db.AssertDBExists(t, "saved_filters", map[string]interface{}{
		"id":          1,
		"title":       "NewTitle",
		"description": "",
	})
}

func TestSavedFilter_Delete(t *testing.T) {
	db.LoadAndAssertFixtures(t)
	sf := &SavedFilter{
		ID: 1,
	}
	err := sf.Delete()
	assert.NoError(t, err)
	db.AssertDBMissing(t, "saved_filters", map[string]interface{}{
		"id": 1,
	})
}
